

make.beastxml.start <- function(data.comment) {
  catln(0, "<?xml version=\"1.0\" standalone=\"yes\"?>")
  commentln(data.comment)
  catln(0, "<beast>")
}




make.beastxml.after.patterns <- function(nofdiploids, noftetraploids, noftetratrees, nofgenes, 
                   taxastruct, taxatable,
                   sampledgtrees.fpathbase, sampledmultrees.fpath, sampledparams.fpath,
                   DBUGTUNE.fpath, mulTreeVersion) {
                   
  stopifnot( !(is.null(taxastruct)  &&  is.null(taxatable)) )
  stopifnot( !(!is.null(taxastruct)  &&  !is.null(taxatable)) )   
  
  commentln("Constant size population parameter used to generate starting trees.")
  make.constantSize()
  commentln("Randomly generated starting trees for genes.")
  for (g in 1:nofgenes) {
    make.coalescentTree(g, 0.2)
    }
  commentln("Tree models for gene trees.")
  for (g in 1:nofgenes) {
    make.treeModel(g)
    }
  commentln("Branch models for gene trees.")
  for (g in 1:nofgenes) {
    make.strictClockBranchRate(g)
    }
  commentln("HKY substitution models for gene trees.")
  for (g in 1:nofgenes) {
    make.HKYModel(g)
    }  
  commentln("Site models for gene trees.")
  for (g in 1:nofgenes) {
    make.siteModel(g)
    }   
  commentln("Likelihoods for gene trees.")
  for (g in 1:nofgenes) {
    make.treeLikelihood(g)
    } 
    
  if (mulTreeVersion) {
    commentln("Multiply-labelled species tree model.")    
    initpopval <- 0.2
    commentln("Assignments of sequences to individuals and individuals to species.")
    make.mulSpecies(nofgenes, taxastruct, taxatable)
    commentln("Multiply-labelled species tree, like species tree in *BEAST")    
    make.mulSpeciesTree(initpopval)
    commentln("Yule prior model for multiply-labelled species tree")
    commentln("(contains birthrate parameter)")
    make.yuleMultreePriorModel()
    commentln("Prior probability for multiply-labelled species tree")
    make.mulSpeciesTreePrior()
    commentln("Coalescent for  multiply-labelled species tree: probability")
    commentln("that gene trees fit into the network.")
    make.mulMSCoalescent()
    commentln("Prior for population parameter (vector theta = N*mu)")
    make.multree.mixedDistributionLikelihood(nofdiploids, noftetraploids, noftetratrees, initpopval)
    } else {
    commentln("Species network model.")    
    initpopval <- 0.2
    commentln("Assignments of sequences to individuals and individuals to species.")    
    make.alloppSpecies(nofgenes, taxastruct, taxatable, 0.2)
    commentln("Species network, like species tree in *BEAST")    
    make.alloppSpeciesNetwork(initpopval)
    commentln("Prior model for species network (Yule-like model, one parameter)")
    make.alloppNetworkPriorModel()
    commentln("Prior probability for species network")
    make.apspNetworkPrior()
    commentln("Coalescent for species network: probability that gene trees fit")
    commentln("into the network.")
    make.apspCoalescent()
    commentln("Prior for population parameter (vector theta = N*mu)")
    make.network.mixedDistributionLikelihood(nofdiploids, noftetraploids, noftetratrees, initpopval)
    }

  
  make.operators(nofgenes, mulTreeVersion)
  
  make.mcmc(nofgenes,
          sampledgtrees.fpathbase, sampledmultrees.fpath, 
          sampledparams.fpath, DBUGTUNE.fpath, 
          mulTreeVersion)

  catln(0, "</beast>")
}

                   




make.beastxml.from.sim.network <- function(network, nofgenes, taxa,
                   seqs.fpathbase, sampledgtrees.fpathbase,
                   sampledmultrees.fpath, sampledparams.fpath, DBUGTUNE.fpath,
                   mulTreeVersion) {
  nofdiploids <- count.diploids(network)
  noftetraploids <- count.tetraploids(network)
  noftetratrees <- count.tetratrees(network)
  taxastruct <- taxa.array.to.structure(taxa)

  make.beastxml.start("Simulated data for allopolyploids made by seqgen and R code")
  
  make.taxa.and.alignments(nofgenes, seqs.fpathbase)
  commentln("Patterns")
  for (g in 1:nofgenes) {
    make.patterns(g)
    }

  make.beastxml.after.patterns(nofdiploids, noftetraploids, noftetratrees, nofgenes, 
                   taxastruct, taxatable=NULL,
                   sampledgtrees.fpathbase, sampledmultrees.fpath, sampledparams.fpath,
                   DBUGTUNE.fpath, mulTreeVersion)
}
  
  
  
  
  
  
taxa.array.to.structure <- function(taxa) {
  struct <- NULL
  spp <- collect.unique.string.parts(taxa, 3, 3)
  for (i in 1:length(spp)) {
    taxa.of.spp <- NULL
    for (j in 1:length(taxa)) {
      if (substr(taxa[j], 3, 3) == spp[i]) {
        taxa.of.spp <- c(taxa.of.spp, taxa[j])
      }
    }
    struct <- c(struct, list(taxa.of.spp.to.structure(spp[i], taxa.of.spp))) 
    }
  struct
  }


taxa.of.spp.to.structure <- function(sp, tofs) {
  indivs <- collect.unique.string.parts(tofs, 1, 2)
  sqs <- collect.unique.string.parts(tofs, 4, 4)
  list(sp=sp, indivs=indivs, sqs=sqs)
  } 


collect.unique.string.parts <- function(strings, beg, last) {
  parts <- substr(strings[1], beg, last)
  if (length(strings) > 1) {
    for (i in 2:length(strings)) {
      part <- substr(strings[i], beg, last)
      got = FALSE
      for (j in 1:length(parts)) {
        if (part == parts[j]) { got = TRUE }
        }
      if (!got) {
        parts <- c(parts, part)
        }
      }
    }
  parts
  } 


count.diploids <- function(network) {
  nofdinodes <- length(network$ditree$nodes)
  stopifnot((nofdinodes + 1) %% 2 == 0)
  (nofdinodes + 1)/2
  }
  
count.tetraploids <- function(network) {
  noftetraploids <- 0
  for (i in 1:length(network$tetratrees)) {
    noftetranodes <- length(network$tetratrees[[i]]$nodes)
    stopifnot((noftetranodes + 1) %% 2 == 0)
    noftetraploids <- noftetraploids + (noftetranodes + 1)/2   
    }
  noftetraploids
  }

count.tetratrees <- function(network) {
  length(network$tetratrees)
  }


  
# Read in the sequences files and join to make some XML
make.taxa.and.alignments <- function(nofgenes, seqs.fpathbase) {
commentln("Taxa. Each taxon is a sequence (labelled A,B,...) from")
commentln("an individual (01,02,03,...) in a species (a,b,c...).")
  for (g in 1:nofgenes) {
    numastext <- as.character(g)
    textlines <- readLines(paste(seqs.fpathbase, numastext, ".txt", sep=""))
    n <- length(textlines)
  
    tokens <- strsplit(textlines, split=" +")
  
    if (g == 1) {
      taxa <- rep("", n-1)
      for (i in 2:n) {
        taxa[i-1] <- tokens[[i]][1]
        }
      taxa <- sort(taxa)
      
      cat("\t<taxa id=\"taxa\">\n")
      for (i in 2:n) {
        cat("\t\t<taxon id=\"", taxa[i-1], "\"/>\n", sep="")
        }
      cat("\t</taxa>\n")
      } else {
      # error check would be good
      }
    commentln(paste("Simulated alignment for gene", g, "of", nofgenes, "genes."))
    cat("\t<alignment id=\"alignment", numastext, "\" dataType=\"nucleotide\">\n", sep="")
    for (i in 2:n) {
      taxon <- tokens[[i]][1]
      seqnce <- tokens[[i]][2]
      cat("\t\t<sequence>\n", sep="")
      cat("\t\t\t<taxon idref=\"", taxon, "\" />\n", sep="")
      cat(seqnce, "\n", sep="")
      cat("\t\t</sequence>\n", sep="")
      }
    cat("\t</alignment>\n")
    }
  }






make.operators <- function(nofgenes, mulTreeVersion) { 
  commentln("Operators for gene trees.")
  catln(1, "<operators", id("operators"), ">")
  for (g in 1:nofgenes) {
    make.scaleOperator.kappa(g, weight=1)
    }    
  for (g in 1:nofgenes) {
    make.deltaExchange.frequencies(g, weight=1)
    }
  for (g in 1:nofgenes) {
    if (g > 1) {
      make.scaleOperator.clockrate(g, weight=1)
      }
    }
  for (g in 1:nofgenes) {
    make.subtreeSlide.treeModel(g, weight=15) 
    }
  for (g in 1:nofgenes) {
    make.narrowExchange.treeModel(g, weight=15) 
    }    
  for (g in 1:nofgenes) {
    make.wideExchange.treeModel(g, weight=3) 
    }    
  for (g in 1:nofgenes) {
    make.wilsonBalding.treeModel(g, weight=3) 
    }    
  for (g in 1:nofgenes) {
    make.scaleOperator.treeModel.rootHeight(g, weight=3) 
    }    
  for (g in 1:nofgenes) {
    make.uniformOperator.treeModel.internalNodeHeights(g, weight=30) 
    }    
  # this moves the inode heights plus the clock rates relative 
  # to the first gene - note g==1 case done differently
  for (g in 1:nofgenes) {
    make.upDownOperator.clockrate.allInternalNodeHeights(g, weight=30) 
    }    

  if (mulTreeVersion) {
    commentln("Operator for both multree and gene trees,stretches/squeezes everything.")
    make.mulTree.upDownOperator.clocks.pops.heights(nofgenes, weight=30)
    commentln("Operators for MUL-tree.")
    make.scaleOperator.species.popMean(weight=5+nofgenes)
    make.scaleOperator.mulSpeciesTree.splitPopSize(weight=30+6*nofgenes)
    make.scaleOperator.mulSpecies.prior.birthRate(weight=30)
    make.mulTreeNodeReHeightOperator(weight=30+15*nofgenes)
    make.mulTreesequenceReassignmentOperator(weight=30+30*nofgenes)
    } else {
    commentln("Operator for both network and gene trees,stretches/squeezes everything.")
    make.network.upDownOperator.clocks.pops.heights(nofgenes, weight=30)
    commentln("Operators for network.")
    make.scaleOperator.species.popMean(weight=5+nofgenes)
    make.scaleOperator.apspnetwork.splitPopSize(weight=30+6*nofgenes) 
    make.scaleOperator.apspnetwork.prior.eventRate(weight=30) 
    make.networkNodeReHeightOperator(weight=30+15*nofgenes)
    make.sequenceReassignmentOperator(weight=30+30*nofgenes)   
    make.moveLegsOperator(weight=30+15*nofgenes) 
    } 
  catln(1, "</operators>")
  }
  

 
  
make.logs <- function(nofgenes,
                      sampledgtrees.fpathbase, sampledmultrees.fpath,
                      sampledparams.fpath, DBUGTUNE.fpath, 
                      mulTreeVersion) {
  commentln("Log to screen.")
  catln(2, "<log", id("screenLog"), namevalue("logEvery", beast.screen.logevery), ">") 
  catln(3, "<column", namevalue("label", "Posterior"), namevalue("dp", 4),
                       namevalue("width", 12), ">")
  catln(3, idref("posterior", "posterior"))
  catln(3, "</column>")
  catln(3, "<column", namevalue("label", "Prior"), namevalue("dp", 4),
                       namevalue("width", 12), ">")
  catln(3, idref("prior", "prior"))
  catln(3, "</column>")
  catln(3, "<column", namevalue("label", "Likelihood"), namevalue("dp", 4),
                       namevalue("width", 12), ">")
  catln(3, idref("likelihood", "likelihood"))
  catln(3, "</column>")
  catln(3, "<column", namevalue("label", "PopMean"), namevalue("sf", 6),
                       namevalue("width", 12), ">")
  catln(3, idref("parameter", "species.popMean"))
  catln(3, "</column>")
  catln(2, "</log>")
  
  commentln("Main log to file.")
  catln(2, "<log", id("fileLog"), namevalue("logEvery", beast.params.logevery), 
            namevalue("fileName", sampledparams.fpath), ">") 
  catln(3, idref("posterior", "posterior"))
  catln(3, idref("prior", "prior"))
  catln(3, idref("likelihood", "likelihood"))
  if (mulTreeVersion) {
    catln(3, idref("mulSpecies", "mulSpecies"))
    catln(3, idref("mulMSCoalescent", "mulMSCoalescent"))
    catln(3, idref("mixedDistributionLikelihood", "species.popSize"))
    catln(3, idref("mulSpeciesTreePrior", "mulSpeciesTree.prior"))
    catln(3, idref("parameter", "species.popMean"))
    catln(3, idref("parameter", "mulSpeciesTree.splitPopSize"))
    catln(3, idref("parameter", "mulSpecies.yule.birthRate"))
    } else {
    catln(3, idref("alloppSpecies", "alloppSpecies"))
    catln(3, idref("apspCoalescent", "apsp.coalescent"))
    catln(3, idref("mixedDistributionLikelihood", "species.popSize"))
    catln(3, idref("apspNetworkPrior", "apspnetwork.prior"))
    catln(3, idref("parameter", "species.popMean"))
    catln(3, idref("parameter", "apspnetwork.splitPopSize"))
    catln(3, idref("parameter", "apspnetwork.prior.eventRate"))
    }
  for (g in 1:nofgenes) {
    catln(3, numbered.idref("parameter", "", g, ".treeModel.rootHeight"))
    }
  for (g in 1:nofgenes) {
    catln(3, numbered.idref("parameter", "", g, ".kappa"))
    }
  for (g in 1:nofgenes) {
    catln(3, numbered.idref("parameter", "", g, ".frequencies"))
    }
  for (g in 1:nofgenes) {
    catln(3, numbered.idref("parameter", "", g, ".clock.rate"))
    }
  for (g in 1:nofgenes) {
    catln(3, numbered.idref("treeLikelihood", "", g, ".treeLikelihood"))
    }
  catln(2, "</log>")
  
  commentln("Gene tree log files.")
  for (g in 1:nofgenes) {
    fname <- paste(sampledgtrees.fpathbase, g, ".txt", sep="")
    catln(2, "<logTree", numbered.id("", g, ".treeFileLog"), namevalue("logEvery", beast.gtrees.logevery), 
              namevalue("fileName", fname), 
              namevalue("sortTranslationTable", "true"),
              namevalue("nexusFormat", "true"), ">") 
    catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
    catln(2, "</logTree>")
    }
  
  commentln("MUL-tree log file.")
  catln(2, "<logTree", id("multreeFileLog"), namevalue("logEvery", beast.multree.logevery), 
         namevalue("fileName", sampledmultrees.fpath), 
         namevalue("sortTranslationTable", "true"),
         namevalue("nexusFormat", "true"), ">") 
  if (mulTreeVersion) {
    catln(3, idref("mulSpeciesTree", "mulSpeciesTree"))
    } else {
    catln(3, idref("alloppSpeciesNetwork", "apspnetwork"))
    }   
  catln(2, "</logTree>")    

  if (!is.null(DBUGTUNE.fpath)) {
    commentln("Debugging and tuning log.")
    catln(2, "<log", id("alloppDBUGTUNE"), namevalue("logEvery", beast.dbugtune.logevery), 
                      namevalue("fileName", DBUGTUNE.fpath), ">")
  if (mulTreeVersion) {
    catln(3, idref("mulSpeciesTree", "mulSpeciesTree"))
    } else {
    catln(3, idref("alloppSpeciesNetwork", "apspnetwork"))
    }   
    catln(2, "</log>")
    }
  } 


  
 
make.mcmc <- function(nofgenes, sampledgtrees.fpathbase,
                   sampledmultrees.fpath, sampledparams.fpath, DBUGTUNE.fpath, 
                   mulTreeVersion) { 
  catln(1, "<mcmc", id("mcmc"), namevalue("chainLength", beast.chain.length),
                       namevalue("autoOptimize", "true") , ">")
  catln(2, "<posterior", id("posterior"), ">")
  catln(3, "<prior", id("prior"), ">") 
  if (mulTreeVersion) {
    catln(4, idref("mulMSCoalescent", "mulMSCoalescent"))
    catln(4, idref("mixedDistributionLikelihood", "species.popSize"))
    catln(4, idref("mulSpeciesTreePrior", "mulSpeciesTree.prior"))
    } else {
    catln(4, idref("apspCoalescent", "apsp.coalescent"))
    catln(4, idref("mixedDistributionLikelihood", "species.popSize"))
    catln(4, idref("apspNetworkPrior", "apspnetwork.prior"))
    }
  for (g in 1:nofgenes) {
    catln(4, "<logNormalPrior", namevalue("mean", 1.0), namevalue("stdev", 1.25),
           namevalue("offset", 0),  namevalue("meanInRealSpace", "false"), ">")
    catln(5, numbered.idref("parameter", "", g, ".kappa"))
    catln(4, "</logNormalPrior>")
    }      
  catln(4, "<oneOnXPrior>")
  catln(5, idref("parameter", "species.popMean")) 
  catln(4, "</oneOnXPrior>")
  catln(4, "<oneOnXPrior>")
  if (mulTreeVersion) {
    catln(5, idref("parameter", "mulSpecies.yule.birthRate"))
    } else {
    catln(5, idref("parameter", "apspnetwork.prior.eventRate"))
    } 
  catln(4, "</oneOnXPrior>")
  for (g in 1:nofgenes) {
    if (g > 1) {
      catln(4, "<gammaPrior", namevalue("shape", 0.1), 
            namevalue("scale", 10), namevalue("offset", 0), ">")
      catln(5, numbered.idref("parameter", "", g, ".clock.rate"))
      catln(4, "</gammaPrior>")
      }
    }
  catln(3, "</prior>")
  
  catln(3, "<likelihood", id("likelihood"), ">")
  for (g in 1:nofgenes) {
    catln(4, numbered.idref("treeLikelihood", "", g, ".treeLikelihood"))
    }
  catln(3, "</likelihood>")
  catln(2, "</posterior>")
  
  catln(2, idref("operators", "operators"))
  
  make.logs(nofgenes, sampledgtrees.fpathbase, sampledmultrees.fpath, 
                      sampledparams.fpath, DBUGTUNE.fpath, 
                      mulTreeVersion)

  catln(1, "</mcmc>")
  }
  
  





